package com.ruoyi.project.mapper;

import com.ruoyi.project.domain.ProjectStaff;

import java.util.List;

public interface ProjectStaffMapper {
    int deleteByPrimaryKey(Long id);

    int insert(ProjectStaff record);

    int insertSelective(ProjectStaff record);

    ProjectStaff selectByPrimaryKey(Long id);

    int updateByPrimaryKeySelective(ProjectStaff record);

    int updateByPrimaryKey(ProjectStaff record);

    void deleteBypid(Long id);

    List<ProjectStaff> selectBypid(Long id);
}
